@@ -1,39 +1,8 @@
 #!/usr/bin/env python
-# ***** BEGIN LICENSE BLOCK *****
-# Version: MPL 1.1/GPL 2.0/LGPL 2.1
-#
-# The contents of this file are subject to the Mozilla Public License Version
-# 1.1 (the "License"); you may not use this file except in compliance with
-# the License. You may obtain a copy of the License at
-# http://www.mozilla.org/MPL/
-#
-# Software distributed under the License is distributed on an "AS IS" basis,
-# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
-# for the specific language governing rights and limitations under the
-# License.
-#
-# The Original Code is Mozilla WebQA Tests.
-#
-# The Initial Developer of the Original Code is Mozilla Foundation.
-# Portions created by the Initial Developer are Copyright (C) 2011
-# the Initial Developer. All Rights Reserved.
-#
-# Contributor(s): Dave Hunt <dhunt@mozilla.com>
-#                 David Burns
-#
-# Alternatively, the contents of this file may be used under the terms of
-# either the GNU General Public License Version 2 or later (the "GPL"), or
-# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
-# in which case the provisions of the GPL or the LGPL are applicable instead
-# of those above. If you wish to allow use of your version of this file only
-# under the terms of either the GPL or the LGPL, and not to allow others to
-# use your version of this file under the terms of the MPL, indicate your
-# decision by deleting the provisions above and replace them with the notice
-# and other provisions required by the GPL or the LGPL. If you do not delete
-# the provisions above, a recipient may use your version of this file under
-# the terms of any one of the MPL, the GPL or the LGPL.
-#
-# ***** END LICENSE BLOCK *****
+
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import cgi
 import base64
@@ -41,11 +10,12 @@ import json
 import os
 import pytest
 import py
+import re
 import time
-import urllib2
 import httplib
 import ConfigParser
 
+import requests
 from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
 from selenium import selenium
 from selenium import webdriver
@@ -55,14 +25,26 @@ import yaml
 def pytest_configure(config):
     if not hasattr(config, 'slaveinput'):
 
+        config.addinivalue_line(
+            'markers', 'nondestructive: mark the test as nondestructive. ' \
+            'Tests are assumed to be destructive unless this marker is ' \
+            'present. This reduces the risk of running destructive tests ' \
+            'accidentally.')
+
         if config.option.base_url:
-            status_code = _get_status_code(config.option.base_url)
-            assert status_code == 200, 'Base URL did not return status code 200. (URL: %s, Response: %s)' % (config.option.base_url, status_code)
+            r = requests.get(config.option.base_url)
+            assert r.status_code == 200, 'Base URL did not return status code 200. (URL: %s, Response: %s)' % (config.option.base_url, r.status_code)
 
         if config.option.webqa_report_path:
             config._html = LogHTML(config)
             config.pluginmanager.register(config._html)
 
+        if not config.option.run_destructive:
+            if config.option.markexpr:
+                config.option.markexpr = 'nondestructive and (%s)' % config.option.markexpr
+            else: 
+                config.option.markexpr = 'nondestructive'
+
 
 def pytest_unconfigure(config):
     html = getattr(config, '_html', None)
@@ -88,6 +70,24 @@ def pytest_runtest_setup(item):
     TestSetup.default_implicit_wait = 10
     item.sauce_labs_credentials_file = item.config.option.sauce_labs_credentials_file
 
+    # consider this environment sensitive if the base url or any redirection
+    # history matches the regular expression
+    r = requests.get(item.config.option.base_url)
+    urls = [h.url for h in r.history] + [r.url]
+    matches = [re.search(item.config.option.sensitive_url, u) for u in urls]
+    sensitive = any(matches)
+
+    destructive = 'nondestructive' not in item.keywords
+
+    if (sensitive and destructive):
+        first_match = matches[next(i for i, match in enumerate(matches) if match)]
+
+        # skip the test with an appropriate message
+        py.test.skip('This test is destructive and the target URL is ' \
+                     'considered a sensitive environment. If this test is ' \
+                     'not destructive, add the \'nondestructive\' marker to ' \
+                     'it. Sensitive URL: %s' % first_match.string)
+
     if item.config.option.api.upper() == 'RC':
         TestSetup.timeout = item.config.option.timeout * 1000
     else:
@@ -230,6 +230,19 @@ def pytest_addoption(parser):
                      metavar='str',
                      help='build identifier (for continuous integration).')
 
+    group = parser.getgroup('safety', 'safety')
+    group._addoption('--sensitiveurl',
+                     action='store',
+                     dest='sensitive_url',
+                     default='mozilla\.(com|org)',
+                     metavar='str',
+                     help='regular expression for identifying sensitive urls.')
+    group._addoption('--destructive',
+                     action='store_true',
+                     dest='run_destructive',
+                     default=False,
+                     help='include destructive tests (tests not explicitly marked as \'nondestructive\'). (disabled by default).')
+
     group = parser.getgroup('credentials', 'credentials')
     group._addoption("--credentials",
                      action="store",
@@ -762,7 +775,7 @@ class LogHTML(object):
         html.append('.passed {color: green}')
         html.append('.skipped, .xfailed {color: orange}')
         html.append('.error, .failed, .xpassed {color: red}')
-        html.append('.log {display: inline-block; width: 800px; height: 230px; overflow-y: scroll; color: black; border: 1px solid #E6E6E6; padding: 5px; background-color: #E6E6E6; font-family: "Courier New", Courier, monospace; white-space: pre}')
+        html.append('.log {display:inline-block; width:800px; max-height: 230px; overflow-y: scroll; color: black; border: 1px solid #E6E6E6; padding: 5px; background-color: #E6E6E6; font-family: "Courier New", Courier, monospace; white-space:pre-wrap}')
         html.append('.screenshot {display: inline-block; border: 1px solid #E6E6E6; width: 320px; height: 240px; overflow: hidden}')
         html.append('.screenshot img {width: 320px}')
         html.append('.video {display: inline-block; width: 320px; height: 240px}')
